/**
 * Copyright (c) 2024 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#include "conversation.h"
#include "assert/assert.h"

void conversationInit(
  conversation_t *conversation,
  const conversationcallback_t init,
  const conversationcallback_t update
) {
  assertNotNull(conversation, "Conversation is NULL.");

  memset(conversation, 0, sizeof(conversation_t));
  conversation->init = init;
  conversation->update = update;
  conversation->index = CONVERSATION_NOT_INITIALIZED;
}

void conversationUpdate(conversation_t *conversation) {
  uint16_t ret;

  assertNotNull(conversation, "Conversation is NULL.");

  // Init or update.
  switch(conversation->index) {
    case CONVERSATION_NOT_INITIALIZED:
      conversation->index = 0;
      ret = conversation->init(
        conversation, conversation->index
      );
      break;
    
    default:
      ret = conversation->update(conversation, conversation->index);
      break;
  }

  // Check ret value and update conversation.
  switch(ret) {
    case CONVERSATION_NEXT:
      conversation->index++;
      conversation->index = conversation->init(
        conversation, conversation->index
      );
      break;
    
    case CONVERSATION_CONTINUE:
      return;
    
    case CONVERSATION_RESTART:
      conversation->index = 0;
      conversation->init(conversation, conversation->index);
      break;
    
    case CONVERSATION_INVALID:
      assertUnreachable("Invalid converstaion retval");
      break;

    case CONVERSATION_DONE:
      break;
    
    default:
      conversation->index = ret;
      conversation->index = conversation->init(
        conversation, conversation->index
      );
      break;
  }
}